# DidwwUps::CreditCard class to handle Credit Cards
#
# Routes:
#    GET        /api/rest/v1/credit_cards/:id/modules(.:format)
#    GET        /api/rest/v1/credit_cards/:credit_card_id/properties(.:format)
#    GET        /api/rest/v1/credit_cards/:credit_card_id/billing_address(.:format)
#    GET        /api/rest/v1/credit_cards(.:format)
#    GET        /api/rest/v1/credit_cards/:id(.:format)
#    DELETE     /api/rest/v1/credit_cards/:id(.:format)
#
# Filter Examples
#   credit_cards = DidwwUps::CreditCard.all(params: { page:1, per_page:10 , q:{token_ref_is_present: true,  cc_expired_gte: '2014-06-01'}})
#   # => GET "/api/rest/v1/credit_cards?page=1&per_page=10&q%5Bcc_number_contains%5D=37"
#
# Possible filter keys for q hash
#     * +created_at_gte+
#     * +created_at_lte+
#     * +cc_expired_gte+
#     * +cc_expired_lte+
#     * +store_id_eq+
#     * +cc_number_contains+
#     * +token_ref_contains+
#     * +token_ref_is_present+  (true)
#     * +token_ref_is_blank+    (true)
#     * +customer_id_eq+
#     * +customer_id_starts_with+
#
# Methods for pagination
#   # credit_cards.total_count
#   # credit_cards.limit_value
#   # credit_cards.offset_value
#
# Find by token example
#  DidwwUps::CreditCard.find("ToTXNnMAUika3o0sS0sqMw")
#  # => GET "/api/rest/v1/credit_cards/ToTXNnMAUika3o0sS0sqMw"
#
# Remove example
#   DidwwUps::CreditCard.new({token_ref:"ToTXNnMAUika3o0sS0sqMw"}, true).destroy
#   # => DELETE "/api/rest/v1/credit_cards/ToTXNnMAUika3o0sS0sqMw"
#
#
module DidwwUps
  class CreditCard < DidwwUps::Base

    self.prefix = "/api/rest/v1/"

    validate do
      unless persisted? # was initialized by DidwwUps::Store#retrieve_credit_card
        self.errors[:base] << "Invalid callback hash" unless self["secure_hash"].to_s == valid_callback_hash
        self.errors[:base] << self["error"] if self["error"].present?
      end
    end

    schema do
      attribute :credit_card_number, :string
      attribute :credit_card_type, :string
      attribute :credit_card_expired, :string
      attribute :unique_id, :string
      attribute :store_id, :integer
      attribute :token_ref, :string
      attribute :updated_at, :datetime
      attribute :customer_id, :string
      attribute :card_id, :string

    end


    def updated_at
      Time.parse(self[:updated_at])
    end

    def initialize(attributes = {}, persisted = false)
      super attributes, persisted #check attributes keys
      if self['success'].to_i == 1
        @attributes['token_ref'] = self['result']
      else
        @attributes['error'] = self['result']
      end
    end

    # Properties
    #  DidwwUps::CreditCard.new({:token_ref => "LQ8IQ4Av6hM3tgX_WLJp9w"}, true).get(:properties)
    #  DidwwUps::CreditCard.find("LQ8IQ4Av6hM3tgX_WLJp9w").properties
    #  # => GET "/api/rest/v1/credit_cards/LQ8IQ4Av6hM3tgX_WLJp9w/properties"
    #  # =>  {"bin"=>"510510", "bank"=>"BANK OF HAWAII", "country"=>"UNITED STATES", "card_brand"=>"MASTERCARD", "card_type"=>"CREDIT", "bank_phone"=>"1-888-643-3888 OR 1-888-643-9888", "customer_ip"=>"195.138.65.189", "fraud_level"=>nil}
    #
    def properties
      attribute_or_get! :properties
    end

    # Payment Modules
    #  DidwwUps::CreditCard.find("LQ8IQ4Av6hM3tgX_WLJp9w").get(:modules)
    #  DidwwUps::CreditCard.find("LQ8IQ4Av6hM3tgX_WLJp9w").modules
    #  # => GET "/api/rest/v1/credit_cards/LQ8IQ4Av6hM3tgX_WLJp9w/modules"
    #  # => ["WorldnetTps"]
    def modules
      self[:modules] ||= self[:modules_tokens].present? ?
          self[:modules_tokens].collect { |t| t.module_name } :
          get(:modules)
    end

    # Billing Address
    #  DidwwUps::CreditCard.new({:token_ref => "LQ8IQ4Av6hM3tgX_WLJp9w"}, true).get(:billing_address)
    #  DidwwUps::CreditCard.find("LQ8IQ4Av6hM3tgX_WLJp9w").billing_address
    #  # => GET "/api/rest/v1/credit_cards/LQ8IQ4Av6hM3tgX_WLJp9w/modules"
    #  #  => "{\"bill_first_name\":\"FirstName\",\"bill_last_name\":\"SecondName\",\"city\":\"Odessa\",\"company\":\"\",\"country\":\"UA\",\"created_at\":\"2014-03-27T12:29:43Z\",\"email\":\"olga@gmail.com\",\"fax_number\":\"\",\"first_address_line\":\"Address line1\",\"id\":1070,\"phone_number\":\"3806600000000\",\"second_address_line\":\"Address line2\",\"state\":null,\"updated_at\":\"2014-03-27T12:29:43Z\",\"zip\":\"65007\",\"avs_result\":null}"
    def billing_address
      attribute_or_get! :billing_address
    end

    # create Payment ,
    # * *See* : DidwwUps::Payment#create
    # * *Returns* :  DidwwUps::Payment
    def create_payment(params={})
      DidwwUps::Payment.create(params.merge({token_ref: self['token_ref']}))
    end

    def to_param
      token_ref
    end

    alias :id :to_param

    def brand
      self[:credit_card_type]
    end

    def display_name
      "#{self.brand} #{self[:credit_card_number]}"
    end

    def expired_this_month?
      self[:credit_card_expired] == DateTime.now.utc.strftime("%Y-%d")
    end

    def expired?
      self[:credit_card_expired] < DateTime.now.utc.strftime("%Y-%d")
    end

    protected
    def valid_callback_hash
      Digest::SHA1.hexdigest "#{self['result']}#{self['success']}#{self['customer_id']}#{self['card_id']}#{self['date_time']}#{DidwwUps.current_store['api_secret_key']}"
    end

    def attribute_or_get!(name)
      if self.attributes.has_key? name
        return {} if  self[name].nil?
        self[name].is_a?(Hash) ? self[name] : self[name].try(:attributes)
      else
        self[name] = get(name)
      end
    end

  end
end