# DidwwUps::Payment class to handle Payments
#
# Routes:
#   GET        /api/rest/v1/payments(.:format)
#   POST      /api/rest/v1/payments(.:format)
#
# Filter Examples
#   payments = DidwwUps::Payment.all(params: { page:1, per_page:10 , q:{success_eq: true}})
#   # => GET "/api/rest/v1/payments?page=1&per_page=10&q%5Bsuccess_eq%5D=true"
#
#  Methods for pagination
#   # payments.total_count
#   # payments.limit_value
#   # payments.offset_value
#
#
# Possible filter keys for params[:q]
#   * +created_at_gte+
#   * +created_at_lte+
#   * +amount_eq+
#   * +cc_number_contains+
#   * +order_id_eq+
#   * +success_eq+
#   * +store_id_eq+
#   * +customer_id_eq+
#
# Create Payment
#
# Example failed by merchant
#
#   payment =  DidwwUps::Payment.create({token_ref: "LQ8IQ4Av6hM3tgX_WLJp9w", order_id: 7, amount: 20.01, operator: 'Admin', description: 'Payment for order # 7'})
#   # => POST "/api/rest/v1/payments" "{\"order_id\":7,\"amount\":20.1,\"operator\":\"Admin\",\"description\":\"Payment for order # 7\",\"token_ref\":\"LQ8IQ4Av6hM3tgX_WLJp9w\"}"
#   # => "{\"id\":1967,\"token_ref\":\"LQ8IQ4Av6hM3tgX_WLJp9w\",\"cc_type\":\"MASTERCARD\",\"cc_number\":\"540400******0001\",\"success\":false,\"error\":\"Declined\",\"amount\":20.5,\"order_id\":7,\"description\":\"Payment for order # 7\",\"operator\":\"Admin\",\"type\":\"WorldnetTps\"}"
#   payment.valid?   # => true
#   payment.success? # => false
#
# Example failed by UPS
#
#   payment =  DidwwUps::Payment.create({token_ref: "LQ8IQ4Av6hM3tgX_WLJp9w", order_id: 6, amount: 20.60, operator: 'Admin', description: 'Payment for order # 6'})
#   # => "{\"errors\":{\"base\":[\"Order was already processed\"]}}"
#   payment.success? # => false
#   payment.valid?   #  => false
#   errors.full_messages  # => ["Order was already processed"]
#
# Example successfull
#
#   payment =  DidwwUps::Payment.create({token_ref: "LQ8IQ4Av6hM3tgX_WLJp9w",order_id: 8, amount: 20.50, operator: 'Admin', description: 'Payment for order # 8'})
#   # => "{\"id\":1968,\"token_ref\":\"LQ8IQ4Av6hM3tgX_WLJp9w\",\"cc_type\":\"MASTERCARD\",\"cc_number\":\"540400******0001\",\"success\":true,\"error\":null,\"amount\":20.5,\"order_id\":8,\"description\":\"Payment for order # 8\",\"operator\":\"Admin\",\"type\":\"WorldnetTps\"}"
#   payment.success?  # => true
#   payment.valid?    # =>  true
#
#
#
module DidwwUps
  class Payment < DidwwUps::Base

    def created_at
      Time.parse(self[:created_at])
    end

    def success?
      valid? && persisted? && success
    end

    #
    # Creates refund from payment
    #
    # Params
    # * +:amount+ - amount of refund,  if not present payment amount is used
    # * +:description+ - Description of refund,  if not present payment description is used
    # * +:operator+ - responsible for refund,  if not present payment operator is used
    #
    #
    def refund(params={})
      params = {amount: self.amount, description: self.description, operator: self.operator}.merge(params).merge({order_id: self.order_id})
      DidwwUps::Refund.create(params)
    end

  end
end